;**************************************************************************
;*
;* FILE  flash_id.asm
;* Copyright (c) 2006 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: flash_id.asm,v 1.2 2006-05-24 21:34:22 tlr Exp $
;*
;* DESCRIPTION
;*   flash_id.asm
;*
;******
	processor 6502


	MAC	sac
	dc.b	$32,{1}
	ENDM
	MAC	sir
	dc.b	$42,{1}
	ENDM
	MAC	bra
	dc.b	$12,{1}-2-.
	ENDM

FLASH_WINDOW	equ	$4000
BUFFER1		equ	$c000
SCALE1		equ	2
BUFFER2		equ	$c100
SCALE2		equ	1

ptr_zp	equ	$fb	
	seg	code
	org	$0801
;**************************************************************************
;*
;* Basic line!
;*
;******
StartOfFile:
	dc.w	EndLine
	dc.w	0
	dc.b	$9e,"2069 /T.L.R/",0
;	        0 SYS2069 /T.L.R/
EndLine:
	dc.w	0

;**************************************************************************
;*
;* SysAddress... When run we will enter here!
;*
;******
SysAddress:
	lda	#<msg
	ldy	#>msg
	jsr	$ab1e
	
	jsr	get_id_pre
	jsr	get_id
	jsr	get_id2
	jsr	get_id_post

	lda	#<msg_atmel
	ldy	#>msg_atmel
	ldx	#>BUFFER1
	jsr	print_buffer
	lda	#<msg_sst
	ldy	#>msg_sst
	ldx	#>BUFFER2
	jsr	print_buffer

	jmp	$a474

msg:	
	dc.b	13,"DTV2 FLASH ID / TLR",13,0
msg_atmel:	
	dc.b	13,"ATMEL METHOD ($C000-$C0FF):",13,0
msg_sst:	
	dc.b	13,"SST METHOD ($C100-$C1FF):",13,0

	
print_buffer:
	stx	ptr_zp+1
	jsr	$ab1e
	lda	#0
	sta	ptr_zp
	tay
	tax
sa_lp1:
	lda	(ptr_zp),y
	jsr	print_hex
	lda	#" "
	jsr	$ffd2
	inx
	cpx	#8
	bne	sa_skp1
	ldx	#0
	lda	#13
	jsr	$ffd2
sa_skp1:
	iny
	cpy	#32
	bne	sa_lp1
	rts
	
;**************************************************************************
;*
;* NAME  print_hex
;*
;* DESCRIPTION
;*   output hex byte.
;*   
;******
print_hex:
	pha
	lsr
	lsr
	lsr
	lsr
	jsr	ph_skp1
	pla
	and	#$0f
ph_skp1:
	clc
	adc	#"0"
	cmp	#"9"+1
	bcc	ph_skp2
	adc	#"A"-("9"+1)-1
ph_skp2:
	jmp	$ffd2

;**************************************************************************
;*
;* NAME  get_id_pre
;*
;* DESCRIPTION
;*   
;******
get_id_pre:
	sei

; charset from something else than ROM.
	lda	#$1f
	sta	$d018

; enable extended registers
	lda	#1
	sta	$d03f
; Important! Set DMA source address to RAM to stop idle fetches from
; accessing flash.
	lda	#$40
	sta	$d302
	
	
; map in flash, bank 0 at $4000
	sac	$88
	lda	#%01010001
	sac	$dd
	lda	#$00
	sac	$00
	rts

get_id_post:
; map in ram, bank 1 at $4000, and bank 2 at $8000
	sac	$88
	lda	#%01010101
	sac	$dd
	lda	#$01
	sac	$00

; disable extended registers
	lda	#0
	sta	$d03f

; restore charset
	lda	#$14
	sta	$d018	
	cli
	rts	


	align	128
;**************************************************************************
;*
;* NAME  get_id
;*
;* DESCRIPTION
;*   Atmel style id read.
;*   
;******
get_id:
; enter product id
	lda	#$aa
	sta	FLASH_WINDOW+$555*SCALE1
	lda	#$55
	sta	FLASH_WINDOW+$aaa*SCALE1
	lda	#$90
	sta	FLASH_WINDOW+$555*SCALE1

	ldy	#0
gi_lp1:
	lda	FLASH_WINDOW,y
	sta	BUFFER1,y
	iny
	bne	gi_lp1
	

; exit product id
	lda	#$f0
	sta	FLASH_WINDOW

	rts

	align	128
;**************************************************************************
;*
;* NAME  get_id
;*
;* DESCRIPTION
;*   SST style id read.
;*   
;******
get_id2:
; enter product id
	lda	#$aa
	sta	FLASH_WINDOW+$aaa*SCALE2
	lda	#$55
	sta	FLASH_WINDOW+$555*SCALE2
	lda	#$90
	sta	FLASH_WINDOW+$aaa*SCALE2

	ldy	#0
gi2_lp1:
	lda	FLASH_WINDOW,y
	sta	BUFFER2,y
	iny
	bne	gi2_lp1
	

; exit product id
	lda	#$f0
	sta	FLASH_WINDOW

	rts
; eof
